@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: servers.py 1172 2013-02-19 10:34:22Z g.rodola $
+# $Id: servers.py 1203 2013-04-05 11:54:58Z g.rodola $
 
 #  ======================================================================
 #  Copyright (C) 2007-2013 Giampaolo Rodola' <g.rodola@gmail.com>
@@ -68,6 +68,7 @@ import errno
 import select
 import logging
 import signal
+import time
 
 from pyftpdlib.log import logger
 from pyftpdlib.ioloop import Acceptor, IOLoop
@@ -161,7 +162,9 @@ class FTPServer(Acceptor):
                                      self.handler.passive_ports[-1])
         else:
             pasv_ports = None
-        logger.info(">>> starting FTP server on %s:%s <<<" % self.address)
+        addr = self.address
+        logger.info(">>> starting FTP server on %s:%s, pid=%i <<<"
+                    % (addr[0], addr[1], os.getpid()))
         logger.info("poller: %r", self.ioloop.__class__)
         logger.info("masquerade (NAT) address: %s",
                     self.handler.masquerade_address)
@@ -316,11 +319,25 @@ class _SpawnerBase(FTPServer):
             poll_timeout = getattr(self, 'poll_timeout', None)
             soonest_timeout = poll_timeout
 
-            while socket_map and not self._exit.is_set():
+            while (socket_map or tasks) and not self._exit.is_set():
                 try:
-                    poll(timeout=soonest_timeout)
+                    if socket_map:
+                        poll(timeout=soonest_timeout)
                     if tasks:
                         soonest_timeout = sched_poll()
+                        # Handle the case where socket_map is emty but some
+                        # cancelled scheduled calls are still around causing
+                        # this while loop to hog CPU resources.
+                        # In theory this should never happen as all the sched
+                        # functions are supposed to be cancel()ed on close()
+                        # but by using threads we can incur into
+                        # synchronization issues such as this one.
+                        # https://code.google.com/p/pyftpdlib/issues/detail?id=245
+                        if not socket_map:
+                            ioloop.sched.reheapify() # get rid of cancel()led calls
+                            soonest_timeout = sched_poll()
+                            if soonest_timeout:
+                                time.sleep(min(soonest_timeout, 1))
                     else:
                         soonest_timeout = None
                 except (KeyboardInterrupt, SystemExit):
