@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: test_contrib.py 1171 2013-02-19 10:13:09Z g.rodola $
+# $Id: test_contrib.py 1188 2013-03-05 23:15:13Z g.rodola $
 
 #  pyftpdlib is released under the MIT license, reproduced below:
 #  ========================================================================
@@ -278,21 +278,27 @@ class TestFTPS(unittest.TestCase):
         # secured
         self.client.prot_p()
         sock = self.client.transfercmd('list')
-        sock.settimeout(TIMEOUT)
-        while 1:
-            if not sock.recv(1024):
-                self.client.voidresp()
-                break
-        self.assertTrue(isinstance(sock, ssl.SSLSocket))
-        # unsecured
-        self.client.prot_c()
+        try:
+            sock.settimeout(TIMEOUT)
+            while 1:
+                if not sock.recv(1024):
+                    self.client.voidresp()
+                    break
+            self.assertTrue(isinstance(sock, ssl.SSLSocket))
+            # unsecured
+            self.client.prot_c()
+        finally:
+            sock.close()
         sock = self.client.transfercmd('list')
-        sock.settimeout(TIMEOUT)
-        while 1:
-            if not sock.recv(1024):
-                self.client.voidresp()
-                break
-        self.assertFalse(isinstance(sock, ssl.SSLSocket))
+        try:
+            sock.settimeout(TIMEOUT)
+            while 1:
+                if not sock.recv(1024):
+                    self.client.voidresp()
+                    break
+            self.assertFalse(isinstance(sock, ssl.SSLSocket))
+        finally:
+            sock.close()
 
     def test_feat(self):
         feat = self.client.sendcmd('feat')
@@ -827,9 +833,10 @@ def test_main():
     for test in tests:
         test_suite.addTest(unittest.makeSuite(test))
     try:
-        unittest.TextTestRunner(verbosity=2).run(test_suite)
+        result = unittest.TextTestRunner(verbosity=2).run(test_suite)
     finally:
         cleanup()
+    return result
 
 if __name__ == '__main__':
-    test_main()
+    sys.exit(not test_main().wasSuccessful())
